// Importing necessary D3 modules
import {
  select,
  csv,
  scaleLinear,
  scaleBand,
  extent,
  axisLeft,
  axisBottom,
  line,
  curveBasis,
} from 'd3';

// Selecting the SVG element from the DOM
const svg = select('svg');

// Getting the width and height of the SVG element
const width = +svg.attr('width');
const height = +svg.attr('height');

// Render function to create the line chart
const render = (data) => {
  // Define the accessor functions for x and y values
  const xValue = (d) => d.model;
  const xAxisLabel = 'Car Model'; // X-axis label

  const yValue = (d) => d.mpg;
  const circleRadius = 2;
  const yAxisLabel = 'Miles Per Gallon'; // Y-axis label

  // Define the margin for the chart
  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 105,
  };

  // Calculate inner width and height based on margins
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Create x and y scales
  const xScale = scaleBand()
    .domain(data.map(xValue))
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  // Create a group element for the chart and apply margins
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Create x and y axes
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(15);

  // Append y-axis to the chart
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove(); // Remove y-axis line

  // Append y-axis label
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -70)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  // Append x-axis to the chart
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG.select('.domain').remove(); // Remove x-axis line

  // Rotate x-axis labels
  xAxisG
    .selectAll('.tick text')
    .attr('transform', 'rotate(-45)')
    .attr('text-anchor', 'end');

  // Append x-axis label
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 50)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);

  // Create a line generator for the line chart
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)) + xScale.bandwidth() / 2)
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis); // Use curveBasis for smooth lines

  // Append the line to the chart
  g.append('path')
    .attr('class', 'line-path')
    .attr('d', lineGenerator(data));

  // Append chart title
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text('Miles Per Gallon (mpg) by Car Model');
};

// URL of the dataset
const csvUrl =
  'https://gist.githubusercontent.com/NandithaSura/784eef9bc929bc7f7eae9627b8fd74d4/raw/2654abeb912711ee2a3741ecad4d03240fa850bb/ice7.csv';

// Load the dataset and call the render function
csv(csvUrl).then((data) => {
  render(data);
});
